import datetime
import os
import random
import time
import warnings
import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
from IPython.display import clear_output

# Ignore log of 0 warnings
warnings.simplefilter("ignore")

input_columns=['time', 'bugname', 'strategy', 'numpaths', 'status']

# Force these columns to numeric
numeric_cols = ['time', 'numpaths']

# Pointer to time
time_col = input_columns[0]

# Pointer to bugname
bug_col = input_columns[1]

# Pointer to strategy
str_col = input_columns[2]

# Pointer to numpath
numpaths_col = input_columns[3]

# Pointer to status
status_col = input_columns[4]

def ceildiv(a, b):
    return -(-a // b)

def bar_plot(df_input, ax):
    bug_numbers = df_input[bug_col].unique()
    strategies = df_input[str_col].unique()
    
    data_done = df_input.loc[(df_input[status_col] == 'Done') | (df_input[time_col] == 0)].drop_duplicates(subset=[bug_col, str_col], keep='last')
    data_done.bugname = data_done.bugname.astype("category").cat.set_categories(bug_numbers)
    data_done.strategy = data_done.strategy.astype("category").cat.set_categories(strategies)
    data_done = data_done.sort_values([bug_col, str_col])
    
    x = np.arange(len(data_done[bug_col].unique()))

    bar_width = 0.08
    multiplier = 0
    for strategy in data_done[str_col].unique():
        ax.bar((x + (bar_width * multiplier)), data_done.loc[data_done[str_col] == strategy, numpaths_col], width = bar_width, label = strategy)
        multiplier += 1

    # Fix the x-axes.
    ax.set_xticks(x + (bar_width * len(data_done.strategy.unique())) / 2)
    ax.set_xticklabels(data_done.bugname.unique())
    
    # Toggle y-axes to log-scale
    ax.set_yscale('log')

    # Add legend.
    ax.legend()

    # Axis styling.
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.spines['left'].set_visible(False)
    ax.spines['bottom'].set_color('#DDDDDD')
    ax.tick_params(bottom=False, left=False)
    ax.set_axisbelow(True)
    ax.yaxis.grid(True, color='#EEEEEE')
    ax.xaxis.grid(False)

    # Add axis and chart labels.
    ax.set_xlabel('Bug Name', labelpad=15)
    ax.set_ylabel('# Path Explored', labelpad=15)
    ax.set_title('Path explored for each bugs by each strategy', pad=15)
    
def line_plot(df_bug, bug, ax):
    strategies = df_bug[str_col].unique()
    
    for strategy in strategies:
        ax.plot(df_bug.loc[df_bug[str_col] == strategy][time_col], df_bug.loc[df_bug[str_col] == strategy][numpaths_col], label = strategy)
        
    # Toggle y-axes to log-scale
    ax.set_yscale('log')

    # Add legend.
    ax.legend()

    # Axis styling.
    ax.spines['top'].set_visible(False)
    ax.spines['right'].set_visible(False)
    ax.spines['left'].set_visible(False)
    ax.spines['bottom'].set_color('#DDDDDD')
    ax.tick_params(bottom=False, left=False)
    ax.set_axisbelow(True)
    ax.yaxis.grid(True, color='#EEEEEE')
    ax.xaxis.grid(False)

    # Add axis and chart labels.
    ax.set_xlabel('Time (ms)', labelpad=15)
    ax.set_ylabel('Path Numbers', labelpad=15)
    ax.set_title('Bug ' + bug, pad=15)

def live_plot(df_input):
    clear_output(wait=True)
    bug_numbers = df_input[bug_col].unique()
    
    # Define subplots with grids and size according to bug numbers     
    fig, ax = plt.subplots(ceildiv((len(bug_numbers) + 1), 3), 3, figsize=(50, (ceildiv((len(bug_numbers) + 1), 3)*15)))
    axn = ax.flatten()
    
    # Plot line charts     
    for i in range(len(bug_numbers)):
        bug = bug_numbers[i]
        df = df_input.loc[df_input[bug_col] == bug]
        curr_ax = axn[i]
        
        line_plot(df, bug, curr_ax)
    
    # Plot bar chart on latest subplot     
    bar_plot(df_input, axn[len(bug_numbers)])
    
    # Hide unused subplots
    for i in range(len(bug_numbers) + 1, len(axn)):
        axn[i].axis('off')
    
    # Adjust and show all the subplots
    fig.tight_layout()
    plt.show()
